import {
	type Editor,
	type MarkdownView,
	type MarkdownFileInfo,
	type App,
	Notice,
	TFile,
} from "obsidian";
import { EditorCheckCommand } from "./Command";
import type { BookTrackerPluginSettings } from "@ui/settings";
import { ReadingProgressModal } from "@ui/modals";
import type { ReadingLog } from "@utils/ReadingLog";

export class LogReadingProgressCommand extends EditorCheckCommand {
	constructor(
		private readonly app: App,
		private readonly readingLog: ReadingLog,
		private readonly settings: BookTrackerPluginSettings
	) {
		super("log-reading-progress", "Log Reading Progress");
	}

	private getPageCount(file: TFile): number {
		return (
			(this.app.metadataCache.getFileCache(file)?.frontmatter?.[
				this.settings.pageCountProperty
			] as number | undefined) ?? 0
		);
	}

	protected check(
		_editor: Editor,
		ctx: MarkdownView | MarkdownFileInfo
	): boolean {
		return !(
			ctx.file === null ||
			this.settings.pageCountProperty === "" ||
			this.getPageCount(ctx.file) <= 0
		);
	}

	protected async run(
		_editor: Editor,
		ctx: MarkdownView | MarkdownFileInfo
	): Promise<void> {
		const file = ctx.file!;
		const fileName = file.basename;
		const pageCount = this.getPageCount(file);
		const pageNumber = await ReadingProgressModal.createAndOpen(
			this.app,
			pageCount
		);

		if (pageNumber <= 0 || pageNumber > pageCount) {
			new Notice(
				`Invalid page number: ${pageNumber}. It must be between 1 and ${pageCount}.`
			);
			return;
		}

		try {
			await this.readingLog.createEntry(fileName, pageNumber, pageCount);
		} catch (error) {
			new Notice(
				`Failed to log reading progress for ${fileName}: ${error}`
			);
			return;
		}

		new Notice(
			`Logged reading progress for ${fileName}: Page ${pageNumber} of ${pageCount}.`
		);
	}
}
